#include <unistd.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/sysinfo.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <memory>
#include <iostream>
#include <thread>
#include <list>
#include <mutex>
#include <vector>
#include <chrono>
#include <algorithm>
#include <opencv2/core.hpp>
#include <opencv2/imgproc.hpp>
#include <opencv2/imgcodecs.hpp>
#include <opencv2/highgui.hpp>
#include <iostream>
#include "tof_sdk_typedef.h"
#include "tof_dev_sdk.h"

// ----------------
// Utility function
// ----------------

struct StreamCallBackParam{
  TofDeviceInfo struCaps;

  UINT32 tof_frame_count;
  UINT32 rgb_frame_count;
  
  cv::Mat rgb = cv::Mat(1080, 1920, CV_8UC3);
  cv::Mat rgbd = cv::Mat(224,109 , CV_8UC3);
  cv::Mat depth = cv::Mat(224, 109, CV_32FC3);
  cv::Mat gray = cv::Mat(224, 109, CV_8UC1);


};


static void fnTofStream(TofFrameData *tofFrameData, void *pUserData)
{
  // 特别注意：因为有些模块的tofFrameData里的各个指针仅在回调函数内有效，
  // 所以，如果要将tofFrameData数据放到其他线程里去处理，必须将tofFrameData里各个指针对应的内存内容复制一份！！！！！！

  StreamCallBackParam *pStreamParam = (StreamCallBackParam *)pUserData;
  (pStreamParam->tof_frame_count)++;
  //
  const UINT32 width = tofFrameData->frameWidth;
  const UINT32 height = tofFrameData->frameHeight;

  cv::Mat gray(height, width, CV_8UC1, tofFrameData->pGrayData);
  cv::Mat rgbd(height, width, CV_8UC3, tofFrameData->pRgbD);
  cv::Mat depth(height, width, CV_32FC3, tofFrameData->pPointData);
  pStreamParam->gray = gray;
  pStreamParam->rgbd = rgbd;

  // convert depth from 32FC3 to 8UC1, with min max normalization
  // cv::Mat depth(height, width, CV_32FC3, tofFrameData->pPointData);
  // cv::Mat depth_8UC1;
  // min max normalization
  cv::normalize(depth, depth, 0, 255, cv::NORM_MINMAX, CV_8UC3);
  cv::cvtColor(depth, depth, cv::COLOR_BGR2GRAY);
  // pStreamParam->depth = depth_8UC1;
  pStreamParam->depth = depth;
  
  // cv::Mat depth_8UC1;
  // // min max normalization
  // cv::normalize(depth, depth, 0, 255, cv::NORM_MINMAX, CV_8UC1);
  
  return;
}

static void fnRgbStream(RgbFrameData *rgbFrameData, void *pUserData)
{
  // 特别注意：因为有些模块的rgbFrameData里的各个指针仅在回调函数内有效，
  // 所以，如果要将rgbFrameData数据放到其他线程里去处理，必须将rgbFrameData里各个指针对应的内存内容复制一份！！！！！！

  StreamCallBackParam *pStreamParam = (StreamCallBackParam *)pUserData;
  (pStreamParam->rgb_frame_count)++;	
  //
  const UINT32 width = rgbFrameData->frameWidth;
  const UINT32 height = rgbFrameData->frameHeight;

  cv::Mat rgb(height, width, CV_8UC3, rgbFrameData->pFrameData);

  pStreamParam->rgb = rgb;
  return;
}

static void PrintDevInfo(TofDeviceInfo *pTofDeviceInfo)
{
  printf("Dev Info:==================================\n");
  printf(">>  szDevName=%s.\n", pTofDeviceInfo->szDevName);
  printf(">>  szDevId=%s.\n", pTofDeviceInfo->szDevId);
  printf(">>  szFirmwareVersion=%s.\n", pTofDeviceInfo->szFirmwareVersion);
  printf("Dev Info==================================\n\n");
}
static void CallBackTofDeviceStatus(TOFDEV_STATUS tofDevStatus, void *pUserData)
{
	printf("device status: 0x08%x.\n", tofDevStatus);
  pUserData = pUserData;

	if (TOFDEV_STATUS_DEV_BROKEN == tofDevStatus)
	{
		printf("a device connection is broken!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
	}

}
static TofDeviceCapability *GetTofModeCaps(TofDeviceInfo &struCaps, const TOF_MODE tofMode)
{
  for (UINT32 i = 0; ((i < struCaps.capCnt) && (i < TOF_MAX_CAPS_CNT)); i++)
  {
    if (tofMode == struCaps.cap[i].tofMode)
    {
      return (&(struCaps.cap[i]));
    }
  }
  return NULL;
}
// static const char* TofMode2Str(const TOF_MODE mode)
// {
// 	const char* pStr = "Unknown";

// 	switch (mode)
// 	{
// 	case TOF_MODE_STERO_5FPS: pStr = "STERO_5FPS"; break;
// 	case TOF_MODE_STERO_10FPS: pStr = "STERO_10FPS"; break;
// 	case TOF_MODE_STERO_15FPS: pStr = "STERO_15FPS"; break;
// 	case TOF_MODE_STERO_30FPS: pStr = "STERO_30FPS"; break;
// 	case TOF_MODE_STERO_45FPS: pStr = "STERO_45FPS"; break;
// 	case TOF_MODE_STERO_60FPS: pStr = "STERO_60FPS"; break;

// 	case TOF_MODE_MONO_5FPS: pStr = "MONO_5FPS"; break;
// 	case TOF_MODE_MONO_10FPS: pStr = "MONO_10FPS"; break;
// 	case TOF_MODE_MONO_15FPS: pStr = "MONO_15FPS"; break;
// 	case TOF_MODE_MONO_30FPS: pStr = "MONO_30FPS"; break;
// 	case TOF_MODE_MONO_45FPS: pStr = "MONO_45FPS"; break;
// 	case TOF_MODE_MONO_60FPS: pStr = "MONO_60FPS"; break;

// 	case TOF_MODE_HDRZ_5FPS: pStr = "HDRZ_5FPS"; break;
// 	case TOF_MODE_HDRZ_10FPS: pStr = "HDRZ_10FPS"; break;
// 	case TOF_MODE_HDRZ_15FPS: pStr = "HDRZ_15FPS"; break;
// 	case TOF_MODE_HDRZ_30FPS: pStr = "HDRZ_30FPS"; break;
// 	case TOF_MODE_HDRZ_45FPS: pStr = "HDRZ_45FPS"; break;
// 	case TOF_MODE_HDRZ_60FPS: pStr = "HDRZ_60FPS"; break;

// 	case TOF_MODE_5FPS: pStr = "5FPS"; break;
// 	case TOF_MODE_10FPS: pStr = "10FPS"; break;
// 	case TOF_MODE_20FPS: pStr = "20FPS"; break;
// 	case TOF_MODE_30FPS: pStr = "30FPS"; break;
// 	case TOF_MODE_45FPS: pStr = "45FPS"; break;
// 	case TOF_MODE_60FPS: pStr = "60FPS"; break;

// 	case TOF_MODE_ADI_1M5: pStr = "ADI_1M5"; break;
// 	case TOF_MODE_ADI_5M: pStr = "ADI_5M"; break;

// 	case TOF_MODE_CUSTOM_1: pStr = "CUSTOM_1"; break;
// 	case TOF_MODE_CUSTOM_2: pStr = "CUSTOM_2"; break;
// 	case TOF_MODE_CUSTOM_3: pStr = "CUSTOM_3"; break;
// 	case TOF_MODE_CUSTOM_4: pStr = "CUSTOM_4"; break;
// 	case TOF_MODE_CUSTOM_5: pStr = "CUSTOM_5"; break;

// 	case TOF_MODE_DEBUG: pStr = "DEBUG"; break;


// 	default: break;
// 	}

// 	return pStr;
// }
